==> ideaIC-15.0.3-sources.jar: com/intellij/vcs/log/data/VisiblePackBuilder.java <==
/*
 * Copyright 2000-2014 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.intellij.vcs.log.data;

import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.util.Computable;
import com.intellij.openapi.util.Condition;
import com.intellij.openapi.util.Pair;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.util.Function;
import com.intellij.util.containers.ContainerUtil;
import com.intellij.util.containers.HashSet;
import com.intellij.util.ui.UIUtil;
import com.intellij.vcs.log.*;
import com.intellij.vcs.log.graph.GraphCommit;
import com.intellij.vcs.log.graph.PermanentGraph;
import com.intellij.vcs.log.graph.VisibleGraph;
import com.intellij.vcs.log.impl.VcsLogFilterCollectionImpl;
import com.intellij.vcs.log.impl.VcsLogUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;

class VisiblePackBuilder {

  private static final Logger LOG = Logger.getInstance(VisiblePackBuilder.class);

  @NotNull private final VcsLogHashMap myHashMap;
  @NotNull private final Map<Integer, VcsCommitMetadata> myTopCommitsDetailsCache;
  @NotNull private final DataGetter<VcsFullCommitDetails> myCommitDetailsGetter;
  @NotNull private final Map<VirtualFile, VcsLogProvider> myLogProviders;

  VisiblePackBuilder(@NotNull Map<VirtualFile, VcsLogProvider> providers,
                     @NotNull VcsLogHashMap hashMap,
                     @NotNull Map<Integer, VcsCommitMetadata> topCommitsDetailsCache,
                     @NotNull DataGetter<VcsFullCommitDetails> detailsGetter) {
    myHashMap = hashMap;
    myTopCommitsDetailsCache = topCommitsDetailsCache;
    myCommitDetailsGetter = detailsGetter;
    myLogProviders = providers;
  }

  @NotNull
  Pair<VisiblePack, CommitCountStage> build(@NotNull DataPack dataPack,
                                            @NotNull PermanentGraph.SortType sortType,
                                            @NotNull VcsLogFilterCollection filters,
                                            @NotNull CommitCountStage commitCount) {
    VcsLogHashFilter hashFilter = filters.getHashFilter();
    if (hashFilter != null && !hashFilter.getHashes().isEmpty()) { // hashes should be shown, no matter if they match other filters or not
      return Pair.create(applyHashFilter(dataPack, hashFilter.getHashes(), sortType), commitCount);
    }

    Set<Integer> matchingHeads = getMatchingHeads(dataPack.getRefs(), dataPack.getLogProviders().keySet(), filters);
    List<VcsLogDetailsFilter> detailsFilters = filters.getDetailsFilters();
    Collection<Hash> matchingCommits = null;
    boolean canRequestMore = false;
    if (!detailsFilters.isEmpty()) {
      if (commitCount == CommitCountStage.INITIAL) {
        matchingCommits = filterInMemory(dataPack.getPermanentGraph(), detailsFilters, matchingHeads);
        if (matchingCommits.size() < commitCount.getCount()) {
          commitCount = commitCount.next();
          matchingCommits = null;
        }
      }

      if (matchingCommits == null) {
        try {
          matchingCommits = getFilteredDetailsFromTheVcs(myLogProviders, filters, commitCount.getCount());
        }
        catch (VcsException e) {
          // TODO show an error balloon or something else for non-ea guys.
          matchingCommits = Collections.emptyList();
          LOG.error(e);
        }
      }

      canRequestMore = matchingCommits.size() >= commitCount.getCount(); // from VCS: only "==", but from memory can be ">"
    }

    VisibleGraph<Integer> visibleGraph;
    if (matchesNothing(matchingHeads) || matchesNothing(matchingCommits)) {
      visibleGraph = EmptyVisibleGraph.getInstance();
    }
    else {
      visibleGraph = dataPack.getPermanentGraph().createVisibleGraph(sortType, matchingHeads, getMatchedCommitIndex(matchingCommits));
    }
    return Pair.create(new VisiblePack(dataPack, visibleGraph, canRequestMore), commitCount);
  }

  private static <T> boolean matchesNothing(@Nullable Collection<T> matchingSet) {
    return matchingSet != null && matchingSet.isEmpty();
  }

  private VisiblePack applyHashFilter(@NotNull DataPack dataPack,
                                      @NotNull Collection<String> hashes,
                                      @NotNull PermanentGraph.SortType sortType) {
    final Set<Integer> indices = ContainerUtil.map2SetNotNull(hashes, new Function<String, Integer>() {
      @Override
      public Integer fun(String partOfHash) {
        Hash hash = myHashMap.findHashByString(partOfHash);
        return hash != null ? myHashMap.getCommitIndex(hash) : null;
      }
    });
    VisibleGraph<Integer> visibleGraph = dataPack.getPermanentGraph().createVisibleGraph(sortType, null, indices);
    return new VisiblePack(dataPack, visibleGraph, false);
  }

  @Nullable
  private Set<Integer> getMatchingHeads(@NotNull VcsLogRefs refs, @NotNull Set<VirtualFile> roots, @NotNull VcsLogFilterCollection filters) {
    VcsLogBranchFilter branchFilter = filters.getBranchFilter();
    VcsLogRootFilter rootFilter = filters.getRootFilter();
    VcsLogStructureFilter structureFilter = filters.getStructureFilter();

    if (branchFilter == null && rootFilter == null && structureFilter == null) return null;

    Set<Integer> filteredByBranch = null;

    if (branchFilter != null) {
      filteredByBranch = getMatchingHeads(refs, branchFilter);
    }

    Set<Integer> filteredByFile = getMatchingHeads(refs, VcsLogUtil
      .getAllVisibleRoots(roots, rootFilter, structureFilter));

    if (filteredByBranch == null) return filteredByFile;
    if (filteredByFile == null) return filteredByBranch;

    return new HashSet<Integer>(ContainerUtil.intersection(filteredByBranch, filteredByFile));
  }

  private Set<Integer> getMatchingHeads(@NotNull VcsLogRefs refs, @NotNull VcsLogBranchFilter filter) {
    final Collection<String> branchNames = new HashSet<String>(filter.getBranchNames());
    final Collection<String> excludedBranches = new HashSet<String>(filter.getExcludedBranchNames());
    final boolean filterByAcceptance = !filter.getBranchNames().isEmpty();
    return new HashSet<Integer>(ContainerUtil.mapNotNull(refs.getBranches(), new Function<VcsRef, Integer>() {
      @Override
      public Integer fun(@NotNull VcsRef ref) {
        boolean acceptRef = filterByAcceptance ? branchNames.contains(ref.getName()) : !excludedBranches.contains(ref.getName());
        return acceptRef ? myHashMap.getCommitIndex(ref.getCommitHash()) : null;
      }
    }));
  }

  private Set<Integer> getMatchingHeads(@NotNull VcsLogRefs refs, @NotNull Collection<VirtualFile> roots) {
    Set<Integer> result = new HashSet<Integer>();
    for (VcsRef branch : refs.getBranches()) {
      if (roots.contains(branch.getRoot())) {
        result.add(myHashMap.getCommitIndex(branch.getCommitHash()));
      }
    }
    return result;
  }

  @NotNull
  private Collection<Hash> filterInMemory(@NotNull PermanentGraph<Integer> permanentGraph,
                                          @NotNull List<VcsLogDetailsFilter> detailsFilters,
                                          @Nullable Set<Integer> matchingHeads) {
    Collection<Hash> result = ContainerUtil.newArrayList();
    for (GraphCommit<Integer> commit : permanentGraph.getAllCommits()) {
      VcsCommitMetadata data = getDetailsFromCache(commit.getId());
      if (data == null) {
        // no more continuous details in the cache
        break;
      }
      if (matchesAllFilters(data, permanentGraph, detailsFilters, matchingHeads)) {
        result.add(data.getId());
      }
    }
    return result;
  }

  private boolean matchesAllFilters(@NotNull final VcsCommitMetadata commit,
                                    @NotNull final PermanentGraph<Integer> permanentGraph,
                                    @NotNull List<VcsLogDetailsFilter> detailsFilters,
                                    @Nullable final Set<Integer> matchingHeads) {
    boolean matchesAllDetails = ContainerUtil.and(detailsFilters, new Condition<VcsLogDetailsFilter>() {
      @Override
      public boolean value(VcsLogDetailsFilter filter) {
        return filter.matches(commit);
      }
    });
    return matchesAllDetails && matchesAnyHead(permanentGraph, commit, matchingHeads);
  }

  private boolean matchesAnyHead(@NotNull PermanentGraph<Integer> permanentGraph,
                                 @NotNull VcsCommitMetadata commit,
                                 @Nullable Set<Integer> matchingHeads) {
    if (matchingHeads == null) {
      return true;
    }
    // TODO O(n^2)
    int commitIndex = myHashMap.getCommitIndex(commit.getId());
    return ContainerUtil.intersects(permanentGraph.getContainingBranches(commitIndex), matchingHeads);
  }

  @Nullable
  private VcsCommitMetadata getDetailsFromCache(final int commitIndex) {
    VcsCommitMetadata details = myTopCommitsDetailsCache.get(commitIndex);
    if (details != null) {
      return details;
    }
    return UIUtil.invokeAndWaitIfNeeded(new Computable<VcsCommitMetadata>() {
      @Override
      public VcsCommitMetadata compute() {
        return myCommitDetailsGetter.getCommitDataIfAvailable(commitIndex);
      }
    });
  }

  @NotNull
  private static Collection<Hash> getFilteredDetailsFromTheVcs(@NotNull Map<VirtualFile, VcsLogProvider> providers,
                                                               @NotNull VcsLogFilterCollection filterCollection,
                                                               int maxCount) throws VcsException {
    Set<VirtualFile> visibleRoots =
      VcsLogUtil.getAllVisibleRoots(providers.keySet(), filterCollection.getRootFilter(), filterCollection.getStructureFilter());

    Collection<Hash> logs = ContainerUtil.newArrayList();
    for (Map.Entry<VirtualFile, VcsLogProvider> entry : providers.entrySet()) {
      VirtualFile root = entry.getKey();

      if (!visibleRoots.contains(root) ||
          (filterCollection.getUserFilter() != null && filterCollection.getUserFilter().getUserNames(root).isEmpty())) {
        // there is a structure or user filter, but it doesn't match this root
        continue;
      }

      VcsLogFilterCollection rootSpecificCollection = filterCollection;
      if (rootSpecificCollection.getStructureFilter() != null) {
        rootSpecificCollection = replaceStructureFilter(filterCollection, ContainerUtil.newHashSet(VcsLogUtil.getFilteredFilesForRoot(root, filterCollection)));
      }

      List<TimedVcsCommit> matchingCommits = entry.getValue().getCommitsMatchingFilter(root, rootSpecificCollection, maxCount);
      logs.addAll(ContainerUtil.map(matchingCommits, new Function<TimedVcsCommit, Hash>() {
        @Override
        public Hash fun(TimedVcsCommit commit) {
          return commit.getId();
        }
      }));
    }

    return logs;
  }

  @NotNull
  private static VcsLogFilterCollection replaceStructureFilter(@NotNull VcsLogFilterCollection filterCollection,
                                                               @NotNull Set<VirtualFile> files) {
    return new VcsLogFilterCollectionImpl(filterCollection.getBranchFilter(), filterCollection.getUserFilter(),
                                          filterCollection.getHashFilter(), filterCollection.getDateFilter(),
                                          filterCollection.getTextFilter(), new VcsLogStructureFilterImpl(files),
                                          filterCollection.getRootFilter());
  }

  @Nullable
  private Set<Integer> getMatchedCommitIndex(@Nullable Collection<Hash> commits) {
    if (commits == null) {
      return null;
    }

    return ContainerUtil.map2Set(commits, new Function<Hash, Integer>() {
      @Override
      public Integer fun(Hash hash) {
        return myHashMap.getCommitIndex(hash);
      }
    });
  }

}

==> ideaIC-141.3057.6-sources.jar: com/intellij/vcs/log/data/VisiblePackBuilder.java <==
/*
 * Copyright 2000-2014 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.intellij.vcs.log.data;

import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.util.Computable;
import com.intellij.openapi.util.Condition;
import com.intellij.openapi.util.Pair;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.util.Function;
import com.intellij.util.containers.ContainerUtil;
import com.intellij.util.containers.HashSet;
import com.intellij.util.ui.UIUtil;
import com.intellij.vcs.log.*;
import com.intellij.vcs.log.graph.GraphCommit;
import com.intellij.vcs.log.graph.PermanentGraph;
import com.intellij.vcs.log.graph.VisibleGraph;
import com.intellij.vcs.log.impl.VcsLogFilterCollectionImpl;
import com.intellij.vcs.log.impl.VcsLogUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;

class VisiblePackBuilder {

  private static final Logger LOG = Logger.getInstance(VisiblePackBuilder.class);

  @NotNull private final VcsLogHashMap myHashMap;
  @NotNull private final Map<Integer, VcsCommitMetadata> myTopCommitsDetailsCache;
  @NotNull private final CommitDetailsGetter myCommitDetailsGetter;
  @NotNull private final Map<VirtualFile, VcsLogProvider> myLogProviders;

  VisiblePackBuilder(@NotNull Map<VirtualFile, VcsLogProvider> providers,
                     @NotNull VcsLogHashMap hashMap,
                     @NotNull Map<Integer, VcsCommitMetadata> topCommitsDetailsCache,
                     @NotNull CommitDetailsGetter detailsGetter) {
    myHashMap = hashMap;
    myTopCommitsDetailsCache = topCommitsDetailsCache;
    myCommitDetailsGetter = detailsGetter;
    myLogProviders = providers;
  }

  @NotNull
  Pair<VisiblePack, CommitCountStage> build(@NotNull DataPack dataPack,
                                            @NotNull PermanentGraph.SortType sortType,
                                            @NotNull VcsLogFilterCollection filters,
                                            @NotNull CommitCountStage commitCount) {
    VcsLogHashFilter hashFilter = filters.getHashFilter();
    if (hashFilter != null && !hashFilter.getHashes().isEmpty()) { // hashes should be shown, no matter if they match other filters or not
      return Pair.create(applyHashFilter(dataPack, hashFilter.getHashes(), sortType), commitCount);
    }

    Set<Integer> matchingHeads = getMatchingHeads(dataPack.getRefs(), dataPack.getLogProviders().keySet(), filters);
    List<VcsLogDetailsFilter> detailsFilters = filters.getDetailsFilters();
    List<Hash> matchingCommits = null;
    boolean canRequestMore = false;
    if (!detailsFilters.isEmpty()) {
      if (commitCount == CommitCountStage.INITIAL) {
        matchingCommits = filterInMemory(dataPack.getPermanentGraph(), detailsFilters, matchingHeads);
        if (matchingCommits.size() < commitCount.getCount()) {
          commitCount = commitCount.next();
          matchingCommits = null;
        }
      }

      if (matchingCommits == null) {
        try {
          matchingCommits = getFilteredDetailsFromTheVcs(myLogProviders, filters, commitCount.getCount());
        }
        catch (VcsException e) {
          // TODO show an error balloon or something else for non-ea guys.
          matchingCommits = Collections.emptyList();
          LOG.error(e);
        }
      }

      canRequestMore = matchingCommits.size() >= commitCount.getCount(); // from VCS: only "==", but from memory can be ">"
    }

    VisibleGraph<Integer> visibleGraph;
    if (matchesNothing(matchingHeads) || matchesNothing(matchingCommits)) {
      visibleGraph = EmptyVisibleGraph.getInstance();
    }
    else {
      visibleGraph = dataPack.getPermanentGraph().createVisibleGraph(sortType, matchingHeads, getMatchedCommitIndex(matchingCommits));
    }
    return Pair.create(new VisiblePack(dataPack, visibleGraph, canRequestMore), commitCount);
  }

  private static <T> boolean matchesNothing(@Nullable Collection<T> matchingSet) {
    return matchingSet != null && matchingSet.isEmpty();
  }

  private VisiblePack applyHashFilter(@NotNull DataPack dataPack,
                                      @NotNull Collection<String> hashes,
                                      @NotNull PermanentGraph.SortType sortType) {
    final Set<Integer> indices = ContainerUtil.map2SetNotNull(hashes, new Function<String, Integer>() {
      @Override
      public Integer fun(String partOfHash) {
        Hash hash = myHashMap.findHashByString(partOfHash);
        return hash != null ? myHashMap.getCommitIndex(hash) : null;
      }
    });
    VisibleGraph<Integer> visibleGraph = dataPack.getPermanentGraph().createVisibleGraph(sortType, null, indices);
    return new VisiblePack(dataPack, visibleGraph, false);
  }

  @Nullable
  private Set<Integer> getMatchingHeads(@NotNull VcsLogRefs refs, @NotNull Set<VirtualFile> roots, @NotNull VcsLogFilterCollection filters) {
    VcsLogBranchFilter branchFilter = filters.getBranchFilter();
    VcsLogRootFilter rootFilter = filters.getRootFilter();
    VcsLogStructureFilter structureFilter = filters.getStructureFilter();

    if (branchFilter == null && rootFilter == null && structureFilter == null) return null;

    Set<Integer> filteredByBranch = null;

    if (branchFilter != null) {
      filteredByBranch = getMatchingHeads(refs, branchFilter);
    }

    Set<Integer> filteredByFile = getMatchingHeads(refs, VcsLogUtil
      .getAllVisibleRoots(roots, rootFilter, structureFilter));

    if (filteredByBranch == null) return filteredByFile;
    if (filteredByFile == null) return filteredByBranch;

    return new HashSet<Integer>(ContainerUtil.intersection(filteredByBranch, filteredByFile));
  }

  private Set<Integer> getMatchingHeads(@NotNull VcsLogRefs refs, @NotNull VcsLogBranchFilter filter) {
    final Collection<String> branchNames = new HashSet<String>(filter.getBranchNames());
    return new HashSet<Integer>(ContainerUtil.mapNotNull(refs.getBranches(), new Function<VcsRef, Integer>() {
      @Override
      public Integer fun(VcsRef ref) {
        if (branchNames.contains(ref.getName())) {
          return myHashMap.getCommitIndex(ref.getCommitHash());
        }
        return null;
      }
    }));
  }

  private Set<Integer> getMatchingHeads(@NotNull VcsLogRefs refs, @NotNull Collection<VirtualFile> roots) {
    Set<Integer> result = new HashSet<Integer>();
    for (VcsRef branch : refs.getBranches()) {
      if (roots.contains(branch.getRoot())) {
        result.add(myHashMap.getCommitIndex(branch.getCommitHash()));
      }
    }
    return result;
  }

  @NotNull
  private List<Hash> filterInMemory(@NotNull PermanentGraph<Integer> permanentGraph,
                                    @NotNull List<VcsLogDetailsFilter> detailsFilters,
                                    @Nullable Set<Integer> matchingHeads) {
    List<Hash> result = ContainerUtil.newArrayList();
    for (GraphCommit<Integer> commit : permanentGraph.getAllCommits()) {
      VcsCommitMetadata data = getDetailsFromCache(commit.getId());
      if (data == null) {
        // no more continuous details in the cache
        break;
      }
      if (matchesAllFilters(data, permanentGraph, detailsFilters, matchingHeads)) {
        result.add(data.getId());
      }
    }
    return result;
  }

  private boolean matchesAllFilters(@NotNull final VcsCommitMetadata commit,
                                    @NotNull final PermanentGraph<Integer> permanentGraph,
                                    @NotNull List<VcsLogDetailsFilter> detailsFilters,
                                    @Nullable final Set<Integer> matchingHeads) {
    boolean matchesAllDetails = ContainerUtil.and(detailsFilters, new Condition<VcsLogDetailsFilter>() {
      @Override
      public boolean value(VcsLogDetailsFilter filter) {
        return filter.matches(commit);
      }
    });
    return matchesAllDetails && matchesAnyHead(permanentGraph, commit, matchingHeads);
  }

  private boolean matchesAnyHead(@NotNull PermanentGraph<Integer> permanentGraph,
                                 @NotNull VcsCommitMetadata commit,
                                 @Nullable Set<Integer> matchingHeads) {
    if (matchingHeads == null) {
      return true;
    }
    // TODO O(n^2)
    int commitIndex = myHashMap.getCommitIndex(commit.getId());
    return ContainerUtil.intersects(permanentGraph.getContainingBranches(commitIndex), matchingHeads);
  }

  @Nullable
  private VcsCommitMetadata getDetailsFromCache(final int commitIndex) {
    VcsCommitMetadata details = myTopCommitsDetailsCache.get(commitIndex);
    if (details != null) {
      return details;
    }
    return UIUtil.invokeAndWaitIfNeeded(new Computable<VcsCommitMetadata>() {
      @Override
      public VcsCommitMetadata compute() {
        return myCommitDetailsGetter.getCommitDataIfAvailable(commitIndex);
      }
    });
  }

  @NotNull
  private static List<Hash> getFilteredDetailsFromTheVcs(@NotNull Map<VirtualFile, VcsLogProvider> providers,
                                                         @NotNull VcsLogFilterCollection filterCollection,
                                                         int maxCount) throws VcsException {
    Set<VirtualFile> visibleRoots =
      VcsLogUtil.getAllVisibleRoots(providers.keySet(), filterCollection.getRootFilter(), filterCollection.getStructureFilter());

    Collection<List<TimedVcsCommit>> logs = ContainerUtil.newArrayList();
    for (Map.Entry<VirtualFile, VcsLogProvider> entry : providers.entrySet()) {
      VirtualFile root = entry.getKey();

      if (!visibleRoots.contains(root) ||
          (filterCollection.getUserFilter() != null && filterCollection.getUserFilter().getUserNames(root).isEmpty())) {
        // there is a structure or user filter, but it doesn't match this root
        continue;
      }

      VcsLogFilterCollection rootSpecificCollection = filterCollection;
      if (rootSpecificCollection.getStructureFilter() != null) {
        rootSpecificCollection = replaceStructureFilter(filterCollection, ContainerUtil.newHashSet(VcsLogUtil.getFilteredFilesForRoot(root, filterCollection)));
      }

      List<TimedVcsCommit> matchingCommits = entry.getValue().getCommitsMatchingFilter(root, rootSpecificCollection, maxCount);
      logs.add(matchingCommits);
    }

    List<TimedVcsCommit> compoundLog = new VcsLogMultiRepoJoiner<Hash, TimedVcsCommit>().join(logs);
    return ContainerUtil.map(compoundLog, new Function<TimedVcsCommit, Hash>() {
      @Override
      public Hash fun(TimedVcsCommit commit) {
        return commit.getId();
      }
    });
  }

  @NotNull
  private static VcsLogFilterCollection replaceStructureFilter(@NotNull VcsLogFilterCollection filterCollection,
                                                               @NotNull Set<VirtualFile> files) {
    return new VcsLogFilterCollectionImpl(filterCollection.getBranchFilter(), filterCollection.getUserFilter(),
                                          filterCollection.getHashFilter(), filterCollection.getDateFilter(),
                                          filterCollection.getTextFilter(), new VcsLogStructureFilterImpl(files),
                                          filterCollection.getRootFilter());
  }

  @Nullable
  private Set<Integer> getMatchedCommitIndex(@Nullable List<Hash> commits) {
    if (commits == null) {
      return null;
    }

    return ContainerUtil.map2Set(commits, new Function<Hash, Integer>() {
      @Override
      public Integer fun(Hash hash) {
        return myHashMap.getCommitIndex(hash);
      }
    });
  }

}
